@@ -24,6 +24,8 @@
  *   Robert John Churchill   <rjc@netscape.com>
  *   David Hyatt             <hyatt@mozilla.org>
  *   Christopher A. Aillon   <christopher@aillon.com>
+ *   Myk Melez               <myk@mozilla.org>
+ *   Pamela Greene           <pamg.bugs@gmail.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -55,14 +57,19 @@
 
 const DEBUG = false; /* set to false to suppress debug messages */
 
-const SIDEBAR_CONTRACTID        = "@mozilla.org/sidebar;1";
-const SIDEBAR_CID               = Components.ID("{22117140-9c6e-11d3-aaf1-00805f8a4905}");
-const NETSEARCH_CONTRACTID      = "@mozilla.org/rdf/datasource;1?name=internetsearch"
-const nsISupports               = Components.interfaces.nsISupports;
-const nsIFactory                = Components.interfaces.nsIFactory;
-const nsISidebar                = Components.interfaces.nsISidebar;
-const nsIInternetSearchService  = Components.interfaces.nsIInternetSearchService;
-const nsIClassInfo              = Components.interfaces.nsIClassInfo;
+const SIDEBAR_CONTRACTID            = "@mozilla.org/sidebar;1";
+const SIDEBAR_CID                   = Components.ID("{22117140-9c6e-11d3-aaf1-00805f8a4905}");
+const NETSEARCH_CONTRACTID          = "@mozilla.org/rdf/datasource;1?name=internetsearch"
+const nsISupports                   = Components.interfaces.nsISupports;
+const nsIFactory                    = Components.interfaces.nsIFactory;
+const nsISidebar                    = Components.interfaces.nsISidebar;
+const nsISidebar_MOZILLA_1_8_BRANCH = Components.interfaces.nsISidebar_MOZILLA_1_8_BRANCH;
+const nsISidebarExternal            = Components.interfaces.nsISidebarExternal;
+const nsIInternetSearchService      = Components.interfaces.nsIInternetSearchService;
+const nsIClassInfo                  = Components.interfaces.nsIClassInfo;
+
+// File extension for Sherlock search plugin description files
+const SHERLOCK_FILE_EXT_REGEXP = /\.src$/i;
 
 function nsSidebar()
 {
@@ -70,14 +77,22 @@ function nsSidebar()
     const nsIPromptService = Components.interfaces.nsIPromptService;
     this.promptService =
         Components.classes[PROMPTSERVICE_CONTRACTID].getService(nsIPromptService);
+
+    const SEARCHSERVICE_CONTRACTID = "@mozilla.org/browser/search-service;1";
+    const nsIBrowserSearchService = Components.interfaces.nsIBrowserSearchService;
+    this.searchService =
+      Components.classes[SEARCHSERVICE_CONTRACTID].getService(nsIBrowserSearchService);
 }
 
 nsSidebar.prototype.nc = "http://home.netscape.com/NC-rdf#";
 
 function sidebarURLSecurityCheck(url)
 {
-    if (url.search(/(^http:|^ftp:|^https:)/) == -1)
-        throw "Script attempted to add sidebar panel from illegal source";
+    if (!/^(https?:|ftp:)/i.test(url)) {
+        Components.utils.reportError("Invalid argument passed to window.sidebar.addPanel: Unsupported panel URL." );
+        return false;
+    }
+    return true;
 }
 
 /* decorate prototype to provide ``class'' methods and property accessors */
@@ -106,7 +121,9 @@ function (aTitle, aContentURL, aCustomizeURL, aPersist)
                               .getService(Components.interfaces.nsIWindowMediator);
     var win = WINMEDSVC.getMostRecentWindow( "navigator:browser" );
                                                                                 
-    sidebarURLSecurityCheck(aContentURL);
+    if (!sidebarURLSecurityCheck(aContentURL))
+      return;
+
     var dialogArgs = {
       name: aTitle,
       url: aContentURL,
@@ -121,82 +138,123 @@ function (aTitle, aContentURL, aCustomizeURL, aPersist)
                    features, dialogArgs);
 }
 
-/* decorate prototype to provide ``class'' methods and property accessors */
+nsSidebar.prototype.validateSearchEngine =
+function (engineURL, iconURL)
+{
+  try
+  {
+    // Make sure we're using HTTP, HTTPS, or FTP.
+    if (! /^(https?|ftp):\/\//i.test(engineURL))
+      throw "Unsupported search engine URL";
+  
+    // Make sure we're using HTTP, HTTPS, or FTP and refering to a
+    // .gif/.jpg/.jpeg/.png/.ico file for the icon.
+    if (iconURL &&
+        ! /^(https?|ftp):\/\/.+\.(gif|jpg|jpeg|png|ico)$/i.test(iconURL))
+      throw "Unsupported search icon URL.";
+  }
+  catch(ex)
+  {
+    debug(ex);
+    Components.utils.reportError("Invalid argument passed to window.sidebar.addSearchEngine: " + ex);
+    
+    var searchBundle = srGetStrBundle("chrome://browser/locale/search.properties");
+    var brandBundle = srGetStrBundle("chrome://branding/locale/brand.properties");
+    var brandName = brandBundle.GetStringFromName("brandShortName");
+    var title = searchBundle.GetStringFromName("error_invalid_engine_title");
+    var msg = searchBundle.formatStringFromName("error_invalid_engine_msg",
+                                                [brandName], 1);
+    var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"].
+             getService(Components.interfaces.nsIWindowWatcher);
+    ww.getNewPrompter(null).alert(title, msg);
+    return false;
+  }
+  
+  return true;
+}
+
+// The suggestedTitle and suggestedCategory parameters are ignored, but remain
+// for backward compatibility.
 nsSidebar.prototype.addSearchEngine =
 function (engineURL, iconURL, suggestedTitle, suggestedCategory)
 {
-    debug("addSearchEngine(" + engineURL + ", " + iconURL + ", " +
-          suggestedCategory + ", " + suggestedTitle + ")");
+  debug("addSearchEngine(" + engineURL + ", " + iconURL + ", " +
+        suggestedCategory + ", " + suggestedTitle + ")");
 
-    try
-    {
-        // make sure using HTTP (for both engine as well as icon URLs)
+  if (!this.validateSearchEngine(engineURL, iconURL))
+    return;
 
-        if (engineURL.search(/^http:\/\//i) == -1)
-        {
-            debug ("must use HTTP to fetch search engine file");
-            throw Components.results.NS_ERROR_INVALID_ARG;
-        }
+  // OpenSearch files will likely be far more common than Sherlock files, and
+  // have less consistent suffixes, so we assume that ".src" is a Sherlock
+  // (text) file, and anything else is OpenSearch (XML).
+  var dataType;
+  if (SHERLOCK_FILE_EXT_REGEXP.test(engineURL))
+    dataType = Components.interfaces.nsISearchEngine.DATA_TEXT;
+  else
+    dataType = Components.interfaces.nsISearchEngine.DATA_XML;
 
-        if (iconURL.search(/^http:\/\//i) == -1)
-        {
-            debug ("must use HTTP to fetch search icon file");
-            throw Components.results.NS_ERROR_INVALID_ARG;
-        }
+  this.searchService.addEngine(engineURL, dataType, iconURL, true);
+}
 
-        // make sure engineURL refers to a .src file
-        if (engineURL.search(/\.src$/i) == -1)
-        {
-            debug ("engineURL doesn't reference a .src file");
-            throw Components.results.NS_ERROR_INVALID_ARG;
-        }
+// This function exists largely to implement window.external.AddSearchProvider(),
+// to match other browsers' APIs.  The capitalization, although nonstandard here,
+// is therefore important.
+nsSidebar.prototype.AddSearchProvider =
+function (aDescriptionURL)
+{
+  // Get the favicon URL for the current page, or our best guess at the current
+  // page since we don't have easy access to the active document.  Most search
+  // engines will override this with an icon specified in the OpenSearch
+  // description anyway.
+  var WINMEDSVC = Components.classes['@mozilla.org/appshell/window-mediator;1']
+                            .getService(Components.interfaces.nsIWindowMediator);
+  var win = WINMEDSVC.getMostRecentWindow("navigator:browser");
+  var browser = win.document.getElementById("content");
+  var iconURL = "";
+  if (browser.shouldLoadFavIcon(browser.selectedBrowser.currentURI))
+    iconURL = win.gProxyFavIcon.getAttribute("src");
+  
+  if (!this.validateSearchEngine(aDescriptionURL, iconURL))
+    return;
 
-        // make sure iconURL refers to a .gif/.jpg/.jpeg/.png file
-        if (iconURL.search(/\.(gif|jpg|jpeg|png)$/i) == -1)
-        {
-            debug ("iconURL doesn't reference a supported image file");
-            throw Components.results.NS_ERROR_INVALID_ARG;
-        }
+  const typeXML = Components.interfaces.nsISearchEngine.DATA_XML;
+  this.searchService.addEngine(aDescriptionURL, typeXML, iconURL, true);
+}
 
-    }
-    catch(ex)
-    {
-        this.promptService.alert(null, "Failed to add the search engine.");
-        throw Components.results.NS_ERROR_INVALID_ARG;
-    }
+// This function exists to implement window.external.IsSearchProviderInstalled(),
+// for compatibility with other browsers.  It will return an integer value
+// indicating whether the given engine is installed for the current user.
+// However, it is currently stubbed out due to security/privacy concerns
+// stemming from difficulties in determining what domain issued the request.
+// See bug 340604 and
+// http://msdn.microsoft.com/workshop/author/dhtml/reference/methods/issearchproviderinstalled.asp .
+// XXX Implement this!
+nsSidebar.prototype.IsSearchProviderInstalled =
+function (aSearchURL)
+{
+  return 0;
+}
 
-    var titleMessage, dialogMessage;
-    try {
-        var stringBundle = srGetStrBundle("chrome://browser/locale/sidebar/sidebar.properties");
-        if (stringBundle) {
-            titleMessage = stringBundle.GetStringFromName("addEngineConfirmTitle");
-            dialogMessage = stringBundle.GetStringFromName("addEngineConfirmMessage");
-            dialogMessage = dialogMessage.replace(/%title%/, suggestedTitle);
-            dialogMessage = dialogMessage.replace(/%category%/, suggestedCategory);
-            dialogMessage = dialogMessage.replace(/%url%/, engineURL);
-            dialogMessage = dialogMessage.replace(/#/g, "\n");
-        }
-    }
-    catch (e) {
-        titleMessage = "Add Search Engine";
-        dialogMessage = "Add the following search engine?\n\nName: " + suggestedTitle;
-        dialogMessage += "\nSearch Category: " + suggestedCategory;
-        dialogMessage += "\nSource: " + engineURL;
-    }
-          
-    var rv = this.promptService.confirm(null, titleMessage, dialogMessage);
+nsSidebar.prototype.addMicrosummaryGenerator =
+function (generatorURL)
+{
+    debug("addMicrosummaryGenerator(" + generatorURL + ")");
+
+    var stringBundle = srGetStrBundle("chrome://browser/locale/sidebar/sidebar.properties");
+    var titleMessage = stringBundle.GetStringFromName("addMicsumGenConfirmTitle");
+    var dialogMessage = stringBundle.formatStringFromName("addMicsumGenConfirmText", [generatorURL], 1);
       
-    if (!rv)
+    if (!this.promptService.confirm(null, titleMessage, dialogMessage))
         return;
 
-    var internetSearch = Components.classes[NETSEARCH_CONTRACTID].getService();
-    if (internetSearch)    
-        internetSearch = internetSearch.QueryInterface(nsIInternetSearchService);
-    if (internetSearch)
-    {
-        internetSearch.AddSearchEngine(engineURL, iconURL, suggestedTitle,
-                                       suggestedCategory);
-    }
+    var ioService = Components.classes["@mozilla.org/network/io-service;1"].
+                    getService(Components.interfaces.nsIIOService);
+    var generatorURI = ioService.newURI(generatorURL, null, null);
+
+    var microsummaryService = Components.classes["@mozilla.org/microsummary/service;1"].
+                              getService(Components.interfaces.nsIMicrosummaryService);
+    if (microsummaryService)
+      microsummaryService.addGenerator(generatorURI);
 }
 
 // property of nsIClassInfo
@@ -207,7 +265,8 @@ nsSidebar.prototype.classDescription = "Sidebar";
 
 // method of nsIClassInfo
 nsSidebar.prototype.getInterfaces = function(count) {
-    var interfaceList = [nsISidebar, nsIClassInfo];
+    var interfaceList = [nsISidebar, nsISidebar_MOZILLA_1_8_BRANCH, 
+                         nsIClassInfo, nsISidebarExternal];
     count.value = interfaceList.length;
     return interfaceList;
 }
@@ -219,7 +278,9 @@ nsSidebar.prototype.QueryInterface =
 function (iid) {
     if (!iid.equals(nsISidebar) && 
         !iid.equals(nsIClassInfo) &&
-        !iid.equals(nsISupports))
+        !iid.equals(nsISupports) &&
+        !iid.equals(nsISidebar_MOZILLA_1_8_BRANCH) &&
+        !iid.equals(nsISidebarExternal))
         throw Components.results.NS_ERROR_NO_INTERFACE;
     return this;
 }
@@ -249,6 +310,12 @@ function (compMgr, fileSpec, location, type)
                             SIDEBAR_CONTRACTID,
                             true,
                             true);
+                            
+    catman.addCategoryEntry(JAVASCRIPT_GLOBAL_PROPERTY_CATEGORY,
+                            "external",
+                            SIDEBAR_CONTRACTID,
+                            true,
+                            true);
 }
 
 sidebarModule.getClassObject =
